
% (c) 2001 Robin Sommer <rsommer@cs.uni-sb.de>
% 
% You can use it under the terms of the GPL

variable ADSC_SCOREFILE = "";

typedef struct
{
   score,
   group,
   group_re,
   header,
   expires,
   type, 
   next
} Rule;

variable ADSC_READ = 1;
variable ADSC_DELETED = 2;
variable ADSC_UNREAD = 4;
variable ADSC_REPLIED = 8;
variable ADSC_FOLLOWEDUP = 16;

static variable msgs = NULL;

static variable Rules = NULL;

register_hook( "read_article_hook", "adsc_read_article_hook" );
register_hook( "reply_hook", "adsc_reply_hook" );
register_hook( "followup_hook", "adsc_followup_hook" );
register_hook( "article_mode_hook", "adsc_article_mode_hook" );
register_hook( "startup_hook", "adsc_init" );

define adsc_glob2re( pattern )
{
   if( pattern == NULL ) return ".*";
   
   pattern = str_quote_string( pattern, "\\^$[].+?", '\\' );
   ( pattern, ) = strreplace( pattern, "*", ".*", strlen( pattern ) );
   return strcat( "^", strcat( pattern, "\$" ) );
}

define adsc_create_rule( type, score, group, header, expires )
{   
   variable rule = @Rule;
   rule.score = score;
   rule.group = group;
   rule.group_re = adsc_glob2re( group ); 
   rule.header = header;
   rule.type = type;
   rule.expires = expires;
   rule.next = Rules;
   Rules = rule;
}

define adsc_read_article_hook ()
{
   variable id = extract_article_header( "Message-Id" );
   if( id == "" ) return;
   msgs[ id ] |= ADSC_READ;
}

define adsc_reply_hook ()
{
   variable id = extract_article_header( "Message-Id" );
   if( id == "" ) return;
   msgs[ id ] |= ADSC_REPLIED;
}

define adsc_followup_hook ()
{
   variable id = extract_article_header( "Message-Id" );
   if( id == "" ) return;
   msgs[ id ] |= ADSC_FOLLOWEDUP;
}

define adsc_delete ()
{
   variable id = extract_article_header( "Message-Id" );
   if( id == "" ) return;
   msgs[ id ] |= ADSC_DELETED;
   msgs[ id ] &= ~ADSC_READ;
   call( "delete" );
}

define adsc_undelete ()
{
   variable id = extract_article_header( "Message-Id" );
   if( id == "" ) return;
   msgs[ id ] &= ~ADSC_DELETED;
   call( "undelete" );
}

define adsc_article_mode_hook ()
{
   msgs = Assoc_Type[ Integer_Type, 0 ];
}

define adsc_quit ()
{
   variable file, id, status, rule, header, pattern, group, time;
   variable current_group;

   file = fopen( ADSC_SCOREFILE, "a+" );
   
   if( file == NULL ){   
      error( "Can't open adsc score file for writing" );
      return 0;
   }

   current_group = current_newsgroup();
   
   call( "header_bob" );
   
   do {
      
      id = extract_article_header( "Message-Id" );

      if( assoc_key_exists( msgs, id ) ){
         status = msgs[ id ];
         if( status == 0 )
            status = ADSC_UNREAD;
      }
      else {
         status = ADSC_UNREAD;
      }

      foreach( Rules ) using ( "next" ) {
         
         rule = ();
         
         if( string_match( current_group, rule.group_re, 1 ) == 0 )
            continue;
         
         if( status & rule.type ){ % Does the rule the match the article?
            
            if( strup( rule.header ) == "THREAD" ){
               header = "References";
               pattern = id;
               }
            else {
               header = rule.header;
               pattern = extract_article_header( header );
            }

            pattern = str_quote_string( pattern, "\\^$[]*.+?", '\\' );

            if( rule.group == NULL or rule.group == "" )
               group = current_newsgroup();
            else 
               group = rule.group;
            
            time = localtime( _time() + rule.expires * 86400 );
            
            fprintf( file, "[%s]\nScore: %d\nExpires: %d/%d/%d\n%s: %s\n\n",
                     group, rule.score, 
                     time.tm_mon+1, time.tm_mday, 1900+time.tm_year,
                     header, pattern );
         }
      
      }
      
   } while ( header_down(1) );

   fclose( file );
   call( "quit" );

   reload_scorefile(0);
    
}


